package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesWorkCellService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkCenterService;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.mes.api.iservice.base.IProdOrgService;
import cn.estsh.i3plus.mes.api.iservice.busi.IEquipmentService;
import cn.estsh.i3plus.mes.apiservice.util.MesCommConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesArea;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipment;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCell;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCenter;
import cn.estsh.i3plus.pojo.mes.model.ProdOrgExtModel;
import cn.estsh.impp.framework.base.controller.MesBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description:
 * @Reference:
 * @Author: yiming.gu
 * @CreateDate:2019-05-06-9:51
 * @Modify:
 **/
@RestController
@Api("MES组织模型")
@RequestMapping(MesCommonConstant.MES_YANFEN + "/prod-org")
public class ExtProdOrgController extends MesBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(ExtProdOrgController.class);

    @Autowired
    private IProdOrgService prodOrgService;

    @Autowired
    private IEquipmentService mesEquipmentService;

    @Autowired
    private IMesWorkCenterService mesWorkCenterService;

    @Autowired
    private IMesWorkCellService mesWorkCellService;

    @GetMapping(value = "/mes-data-tree/query")
    @ApiOperation(value = "查询MES组织模型数据返回树结构")
    public ResultBean queryMesDataTree() {
        try {
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(prodOrgService.queryMesDataTree(AuthUtil.getOrganize().getOrganizeCode()));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/mes-prod/query")
    @ApiOperation(value = "按条件分页查询MES组织模型数据")
    public ResultBean queryMesProdOrgByPager(ProdOrgExtModel ProdOrgExtModel, Pager pager) {
        try {

            ListPager listPager = null;

            ProdOrgExtModel.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            //工厂
            if (MesEnumUtil.PROD_ORG_LEVEL.LEVEL_ONE.getValue() == ProdOrgExtModel.getLevel()) {

                MesArea mesArea = new MesArea();
                BeanUtils.copyProperties(ProdOrgExtModel, mesArea);

                listPager = prodOrgService.queryMesAreaByPager(mesArea, pager);
            }
            //生产线
            if (MesEnumUtil.PROD_ORG_LEVEL.LEVEL_TWO.getValue() == ProdOrgExtModel.getLevel()) {

                MesWorkCenter workCenter = new MesWorkCenter();
                BeanUtils.copyProperties(ProdOrgExtModel, workCenter);

                listPager = prodOrgService.queryWorkCenterByPager(workCenter, pager);
            }
            //工位
            if (MesEnumUtil.PROD_ORG_LEVEL.LEVEL_THREE.getValue() == ProdOrgExtModel.getLevel()) {

                MesWorkCell workCell = new MesWorkCell();
                BeanUtils.copyProperties(ProdOrgExtModel, workCell);

                listPager = prodOrgService.queryWorkCellByPager(workCell, pager);
            }
            //工位下的设备
            if (MesEnumUtil.PROD_ORG_LEVEL.LEVEL_FOUR.getValue() <= ProdOrgExtModel.getLevel()) {

                MesEquipment equipment = new MesEquipment();
                //这是用到的参数有id 必须使用org.springframework.beans.BeanUtils
                //如果使用org.apache.commons.beanutils.BeanUtils当id为null会默认转化为0
                BeanUtils.copyProperties(ProdOrgExtModel, equipment);

                listPager = mesEquipmentService.queryMesEquipmentByPagerOrg(equipment, pager);
            }

            if (null == listPager) {
                return ResultBean.success("操作成功").setMsg("暂无数据")
                        .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            } else {
                return ResultBean.success("操作成功").setListPager(listPager)
                        .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            }
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/mes-prod/insert")
    @ApiOperation(value = "新增mes组织模型信息(区域，生产线，工位)")
    public ResultBean insertMesProdOrg(ProdOrgExtModel ProdOrgExtModel) {
        try {

            ProdOrgExtModel.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            ConvertBean.serviceModelInitialize(ProdOrgExtModel, AuthUtil.getSessionUser().getUserName());

            prodOrgService.insertMesProdOrg(ProdOrgExtModel);

            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/mes-area/query")
    @ApiOperation(value = "分页查询区域")
    public ResultBean queryMesAreaByPager(MesArea mesArea, Pager pager) {
        try {
            mesArea.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            ListPager areaList = prodOrgService.queryMesAreaByPager(mesArea, pager);

            return ResultBean.success("操作成功").setListPager(areaList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/mes-area/query-all")
    @ApiOperation(value = "按条件查询区域所有数据")
    public ResultBean queryMesAreaByProperties(MesArea mesArea) {
        try {
            mesArea.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            List<MesArea> areaList = prodOrgService.queryMesAreaByProperties(mesArea);

            return ResultBean.success("操作成功").setResultList(areaList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/mes-area/status-update")
    @ApiOperation(value = "根据id启用或禁用区域信息")
    public ResultBean updateMesAreaStatusById(Long id, int status) {
        try {

            if (null == id) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }

            prodOrgService.updateMesAreaStatusById(id, status, AuthUtil.getSessionUser().getUserName(), AuthUtil.getOrganize().getOrganizeCode());

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/mes-area/update")
    @ApiOperation(value = "更新mes区域信息")
    public ResultBean updateMesArea(MesArea mesArea) {
        try {
            //条件验证
            ValidatorBean.beginValid(mesArea)
                    .notNull(MesConstWords.ID, mesArea.getId())
                    .notNull(MesCommConstWords.AREA_CODE, mesArea.getAreaCode())
                    .notNull(MesCommConstWords.AREA_NAME, mesArea.getAreaName());

            prodOrgService.updateMesArea(mesArea, AuthUtil.getSessionUser().getUserName(), AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/work-center/update")
    @ApiOperation(value = "更新生产线信息")
    public ResultBean updateMesWorkCenter(MesWorkCenter workCenter) {
        try {
            //条件验证
            ValidatorBean.beginValid(workCenter)
                    .notNull(MesConstWords.ID, workCenter.getId())
                    .notNull(MesConstWords.WORK_CENTER_CODE, workCenter.getWorkCenterCode())
                    .notNull(MesCommConstWords.WORK_CENTER_NAME, workCenter.getWorkCenterName())
                    .notNull(MesCommConstWords.AREA_CODE, workCenter.getAreaCode());

            mesWorkCenterService.update(workCenter);
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/mes-center/query")
    @ApiOperation(value = "分页查询生产线")
    public ResultBean queryMesWorkCenterByPager(MesWorkCenter workCenter, Pager pager) {
        try {
            workCenter.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            ListPager<MesWorkCenter> workCenterList = prodOrgService.queryWorkCenterByPager(workCenter, pager);

            return ResultBean.success("操作成功").setListPager(workCenterList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/mes-center/query-all")
    @ApiOperation(value = "按条件查询生产线所有数据")
    public ResultBean queryMesWorkCenterByProperties(MesWorkCenter workCenter) {
        try {
            workCenter.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            List<MesWorkCenter> workCenterList = prodOrgService.queryMesWorkCenterByProperties(workCenter);

            return ResultBean.success("操作成功").setResultList(workCenterList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/mes-center-all/query")
    @ApiOperation(value = "按条件查询生产线所有数据")
    public ResultBean queryMesWorkCenterByOrg() {
        try {

            List<MesWorkCenter> workCenterList = prodOrgService.queryMesWorkCenterByOrg(AuthUtil.getOrganize().getOrganizeCode());

            return ResultBean.success("操作成功").setResultList(workCenterList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/mes-center/status-update")
    @ApiOperation(value = "根据id启用或禁用生产线信息")
    public ResultBean updateMesCenterStatusById(Long id, int status) {
        try {

            if (null == id) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }

            prodOrgService.updateMesCenterStatusById(id, status, AuthUtil.getSessionUser().getUserName(), AuthUtil.getOrganize().getOrganizeCode());

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/work-cell/update")
    @ApiOperation(value = "更新工位信息")
    public ResultBean updateMesWorkCell(MesWorkCell workCell) {
        try {
            ValidatorBean.beginValid(workCell)
                    .notNull(MesConstWords.ID, workCell.getId())
                    .notNull(MesConstWords.WORK_CELL_CODE, workCell.getWorkCellCode())
                    .notNull(MesCommConstWords.WORK_CELL_NAME, workCell.getWorkCellName())
                    .notNull(MesConstWords.WORK_CENTER_CODE, workCell.getWorkCenterCode())
                    .notNull("workCellType", workCell.getWorkCellType())
                    .notNull(MesCommConstWords.AREA_CODE, workCell.getAreaCode());

            mesWorkCellService.update(workCell);
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/mes-cell/query")
    @ApiOperation(value = "分页查询工位")
    public ResultBean queryMesWorkCellByPager(MesWorkCell workCell, Pager pager) {
        try {
            workCell.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            ListPager<MesWorkCell> workCellList = prodOrgService.queryWorkCellByPager(workCell, pager);
            return ResultBean.success("操作成功").setListPager(workCellList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/mes-cell/query-all")
    @ApiOperation(value = "按条件查询工位所有数据")
    public ResultBean queryMesWorkCellByProperties(MesWorkCell workCell) {
        try {
            workCell.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            List<MesWorkCell> workCellList = prodOrgService.queryMesWorkCellByProperties(workCell);
            return ResultBean.success("操作成功").setResultList(workCellList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/mes-cell/status-update")
    @ApiOperation(value = "根据id启用或禁用工位信息")
    public ResultBean updateMesCellStatusById(Long id, int status) {
        try {

            if (null == id) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }

            prodOrgService.updateMesCellStatusById(id, status, AuthUtil.getSessionUser().getUserName(), AuthUtil.getOrganize().getOrganizeCode());

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
