package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCustomerCarModelDetailService;
import cn.estsh.i3plus.ext.mes.api.base.IMesCustomerCarModelService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesBom;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomer;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerCarModel;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerRoad;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class MesCustomerCarModelService extends BaseMesService<MesCustomerCarModel> implements IMesCustomerCarModelService {
    protected void setPackQueryBean(MesCustomerCarModel bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getCustomerOrganizeCode(), "customerOrganizeCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCarModelCode(), "carModelCode", packBean);
    }

    protected void onInsertBean(MesCustomerCarModel item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCarModelCode(), "车型不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCarModelCode(), "carModelCode", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getCustomerCode(), "customerCode", roadPackBean);

        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("客户代码【%s】下车型【%s】已经存在，请检查数据", item.getOrganizeCode(),item.getCustomerCode())
                    .build();
        }

    }

    protected void onUpdateBean(MesCustomerCarModel item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCarModelCode(), "车型不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCarModelCode(), "carModelCode", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getCustomerCode(), "customerCode", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("客户代码【%s】下车型【%s】已经存在，请检查数据", item.getOrganizeCode(),item.getCustomerCode())
                    .build();
        }
    }
}
