package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCustomerOriganizeService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerOriganze;
import cn.estsh.i3plus.pojo.mes.repository.MesCustomerCarModelRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author yinxiao
 * @date 2023-2023/6/2 10:54
 * @Description
 */
@Service
@Slf4j
public class MesCustomerOriganizeService extends BaseMesService<MesCustomerOriganze> implements IMesCustomerOriganizeService {

    @Autowired
    private MesCustomerCarModelRepository mesCustomerCarModelRDao;

    protected void onInsertBean(MesCustomerOriganze item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCustomerCode(), "客户代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCustomerCode(), "customerCode", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getCustomerOrganizeCode(), "customerOrganizeCode", roadPackBean);

        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("客户工厂【%s】下客户代码【%s】已经存在，请检查数据", item.getCustomerOrganizeCode(),item.getCustomerCode())
                    .build();
        }

    }

    protected void onUpdateBean(MesCustomerOriganze item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCustomerCode(), "客户代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCustomerCode(), "customerCode", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getCustomerOrganizeCode(), "customerOrganizeCode", roadPackBean);

        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("客户工厂【%s】下客户代码【%s】已经存在，请检查数据", item.getCustomerOrganizeCode(),item.getCustomerCode())
                    .build();
        }
    }


    protected void setPackQueryBean(MesCustomerOriganze bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getCustomerOrganizeCode(), "customerOrganizeCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCustomerOrganizeName(), "customerOrganizeName", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustomerCode(), "customerCode", packBean);

    }

    protected void onDeleteId(Long id) {
        MesCustomerOriganze mesCustomerOriganze = baseRDao.getById(id);
        //校验客户车型配置
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesCustomerOriganze.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesCustomerOriganze.getCustomerOrganizeCode(), "customerOrganizeCode", packBean);
        if(mesCustomerCarModelRDao.isExitByHql(packBean)){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("客户工厂【%s】存在客户车型配置信息，请先删除对应的客户车型配置信息", mesCustomerOriganze.getCustomerOrganizeCode())
                    .build();
        }
    }
}
