package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesLoadingVehiclesOrderService;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.MesPartShippingGroupRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesShippingOrderManagementRepository;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class MesLoadingVehiclesOrderService extends BaseMesService<MesLoadingVehiclesOrder> implements IMesLoadingVehiclesOrderService {


    @Autowired
    private MesLoadingVehiclesOrderDetailService detailService;

    @Autowired
    private MesPartShippingGroupRepository partShippingGroupRepository;
    @Autowired
    private MesShippingOrderManagementRepository shippingOrderManagementRepository;


    @Override
    public MesLoadingVehiclesOrder insert(MesLoadingVehiclesOrder bean) {
        //插入前构造并校验数据
        onInsertBean(bean);

        MesShippingOrderManagement shippingOrderManagement = bean.getShippingOrderNoList().get(0);
        DdlPackBean partShipGroupPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(shippingOrderManagement.getShippingGroupNo(), "shippingGroupNo", partShipGroupPackBean);
        MesPartShippingGroup partShippingGroup = partShippingGroupRepository.getByProperty(partShipGroupPackBean);

        if (StringUtils.isEmpty(partShippingGroup)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组数据为空，请检查数据", shippingOrderManagement.getShippingGroupNo())
                    .build();
        }

        if (StringUtils.isEmpty(partShippingGroup.getIsLoading()) || partShippingGroup.getIsLoading() == 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否有装车不能为空，请检查数据", shippingOrderManagement.getShippingGroupNo())
                    .build();
        }
        if (partShippingGroup.getIsLoading() == MesCommonConstant.FALSE_INTEGER) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否有装车配置为否，则不需要生成装车单，请检查数据", shippingOrderManagement.getShippingGroupNo())
                    .build();
        }

        if (StringUtils.isEmpty(partShippingGroup.getIsFirstInstall()) || partShippingGroup.getIsFirstInstall() == 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否先装车不能为空，请检查数据", shippingOrderManagement.getShippingGroupNo())
                    .build();
        }
        if (StringUtils.isEmpty(partShippingGroup.getIsEnableMemory()) || partShippingGroup.getIsEnableMemory() == 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否启用记忆不能为空，请检查数据", shippingOrderManagement.getShippingGroupNo())
                    .build();
        }

        bean.setIsFirstInstall(partShippingGroup.getIsFirstInstall());
        bean.setIsEnableMemory(partShippingGroup.getIsEnableMemory());

        List<MesLoadingVehiclesOrderDetail> detailList = new ArrayList<>();
        MesLoadingVehiclesOrder finalBean = bean;

        List<String> orderNoList = new ArrayList<>();
        bean.getShippingOrderNoList().forEach(k -> {
            MesLoadingVehiclesOrderDetail detail = new MesLoadingVehiclesOrderDetail();

            orderNoList.add(k.getShippingOrderNo());
            BeanUtils.copyProperties(k, detail, "id");
            detail.setLoadingOrderNo(finalBean.getLoadingOrderNo());
            detail.setWeatherCondition(finalBean.getWeatherCondition());
            detailList.add(detail);
        });

        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getInPackList(orderNoList, "shippingOrderNo", orderPackBean);
        List<MesShippingOrderManagement> orderManagementList = shippingOrderManagementRepository.findByHqlWhere(orderPackBean);

        orderManagementList.forEach(k -> k.setLoadingOrderNo(finalBean.getLoadingOrderNo()));

        String customerCode = bean.getShippingOrderNoList().get(0).getCustomerCode();
        String customerName = bean.getShippingOrderNoList().get(0).getCustomerName();
        String customerOrganizeCode = bean.getShippingOrderNoList().get(0).getCustomerOrganizeCode();
        String customerOrganizeName = bean.getShippingOrderNoList().get(0).getCustomerOrganizeName();
        bean.setSeq(bean.getLoadingOrderNo());
        bean.setStatus(MesExtEnumUtil.LOADING_ORDER_STATUS.CREATE.getValue());
        bean.setCustomerCode(customerCode);
        bean.setCustomerName(customerName);
        bean.setCustomerOrganizeCode(customerOrganizeCode);
        bean.setCustomerOrganizeName(customerOrganizeName);


        shippingOrderManagementRepository.saveAll(orderManagementList);
        detailService.insertBatch(detailList);
        bean = baseRDao.insert(bean);
        return bean;
    }

    @Override
    public void deleteWeaklyByIds(Long[] ids, String userName) {

        for (Long id : ids) {
            MesLoadingVehiclesOrder bean = baseRDao.getById(id);
            if (StringUtils.isEmpty(bean)) continue;
            DdlPackBean detailPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getLoadingOrderNo(), "loadingOrderNo", detailPackBean);
            List<MesLoadingVehiclesOrderDetail> detailDeleteList = detailService.findAll(detailPackBean);
            detailDeleteList.forEach(k -> {
                k.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
            });
            bean.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            ConvertBean.serviceModelUpdate(bean, AuthUtil.getSessionUser().getUserName());
            baseRDao.update(bean);
            detailService.updates(detailDeleteList);
        }
    }


//    @Override
//    public MesLoadingVehiclesOrder update(MesLoadingVehiclesOrder bean) {
//        //更新前构造并校验数据
//        onUpdateBean(bean);
//
//        MesShippingOrderManagement shippingOrderManagement = bean.getShippingOrderNoList().get(0);
//        DdlPackBean partShipGroupPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
//        DdlPreparedPack.getStringEqualPack(shippingOrderManagement.getShippingGroupNo(), "shippingGroupNo", partShipGroupPackBean);
//        MesPartShippingGroup partShippingGroup = partShippingGroupRepository.getByProperty(partShipGroupPackBean);
//
//        if (StringUtils.isEmpty(partShippingGroup)) {
//            throw ImppExceptionBuilder.newInstance()
//                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
//                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
//                    .setErrorDetail("【%s】零件发运组数据为空，请检查数据", shippingOrderManagement.getShippingGroupNo())
//                    .build();
//        }
//
//        if (StringUtils.isEmpty(partShippingGroup.getIsLoading()) || partShippingGroup.getIsLoading() == 0) {
//            throw ImppExceptionBuilder.newInstance()
//                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
//                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
//                    .setErrorDetail("【%s】零件发运组是否有装车不能为空，请检查数据", shippingOrderManagement.getShippingGroupNo())
//                    .build();
//        }
//        if (partShippingGroup.getIsLoading() == MesCommonConstant.FALSE_INTEGER) {
//            throw ImppExceptionBuilder.newInstance()
//                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
//                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
//                    .setErrorDetail("【%s】零件发运组是否有装车配置为否，则不需要生成装车单，请检查数据", shippingOrderManagement.getShippingGroupNo())
//                    .build();
//        }
//
//        if (StringUtils.isEmpty(partShippingGroup.getIsFirstInstall()) || partShippingGroup.getIsFirstInstall() == 0) {
//            throw ImppExceptionBuilder.newInstance()
//                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
//                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
//                    .setErrorDetail("【%s】零件发运组是否先装车不能为空，请检查数据", shippingOrderManagement.getShippingGroupNo())
//                    .build();
//        }
//        if (StringUtils.isEmpty(partShippingGroup.getIsEnableMemory()) || partShippingGroup.getIsEnableMemory() == 0) {
//            throw ImppExceptionBuilder.newInstance()
//                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
//                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
//                    .setErrorDetail("【%s】零件发运组是否启用记忆不能为空，请检查数据", shippingOrderManagement.getShippingGroupNo())
//                    .build();
//        }
//
//    }

    protected void onInsertBean(MesLoadingVehiclesOrder item) {

        List<String> shippingOrderNoList = new ArrayList<>();
        // 数据校验
        ValidatorBean.checkNotNull(item.getLoadingOrderNo(), "装车单号不能为空");
        ValidatorBean.checkNotNull(item.getWeatherCondition(), "天气不能为空");
        ValidatorBean.checkNotNull(item.getShippingOrderNoList(), "发运单不能为空");
        item.getShippingOrderNoList().forEach(k -> {
            if (k.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue() && k.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()) {
                shippingOrderNoList.add(k.getShippingOrderNo());
            }
        });

        if (!CollectionUtils.isEmpty(shippingOrderNoList)) {
            throw ImppExceptionBuilder.newInstance()
                    .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                    .setErrorDetail(String.format("发运单【%s】状态不是【%s】或【%s】,请重新选择数据", shippingOrderNoList, MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue(),
                            MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()))
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .build();
        }

        DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getLoadingOrderNo(), "loadingOrderNo", partPackBean);
//        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", partPackBean);
        boolean flg = baseRDao.isExitByHql(partPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("发运单【%s】已经存在，请检查数据", item.getLoadingOrderNo())
                    .build();
        }

    }

    protected void setPackQueryBean(MesLoadingVehiclesOrder bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getLoadingOrderNo(), "loadingOrderNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustomerCode(), "customerCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustomerOrganizeCode(), "customerOrganizeCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCarNo(), "carNo", packBean);
    }
}
