package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesPartService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderToSapService;
import cn.estsh.i3plus.ext.mes.apiservice.unit.exception.MesException;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.mes.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrderLog;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrderToSap;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.MesWorkOrderToSapRepository;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Service
@Slf4j
public class MesWorkOrderService extends BaseMesService<MesWorkOrder> implements IMesWorkOrderService {
    @Autowired
    private MesWorkOrderLogService workOrderLogService;
    @Autowired
    private ISyncFuncService syncFuncService;
    @Autowired
    private IMesWorkOrderToSapService iMesWorkOrderToSapService;
    @Autowired
    private IMesPartService iMesPartService;

    @Override
    public MesWorkOrder insert(MesWorkOrder bean) {
        //插入前构造数据
        onInsertBean(bean);

        //获取相同类型 相同标识 生产序号最大值的
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(bean.getWorkOrderStatus(), "workOrderStatus", orderPackBean);
        DdlPreparedPack.getNumEqualPack(bean.getOrderFlag(), "orderFlag", orderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"produceSeq"}, orderPackBean);
        MesWorkOrder result = baseRDao.getByProperty(orderPackBean);


        GenSerialNoModel genSerialNoModel = new GenSerialNoModel();
        //若工单类型为排序 则根据排序插入工单数量插入多行 工单数量为1
        if (bean.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
            Integer produceSeq = 0;
            if (null != result && result.getProduceSeq() != null) {
                produceSeq = result.getProduceSeq();
            }

            List<MesWorkOrderLog> orderLogList = new ArrayList<>();
            //生成工单号
            genSerialNoModel = new GenSerialNoModel("ORDER_NO_JIS_SORT");
            genSerialNoModel.setPartNo(bean.getPartNo());
            List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel, bean.getCreateUser(), bean.getOrganizeCode(), 1).getResultList();

            MesWorkOrder copyMesWorkOrder = new MesWorkOrder();
            BeanUtils.copyProperties(bean, copyMesWorkOrder);
            copyMesWorkOrder.setProduceSeq(produceSeq + 1);
            copyMesWorkOrder.setQty(1D);
            copyMesWorkOrder.setOrderNo(resultList.get(0));
            ConvertBean.serviceModelInitialize(copyMesWorkOrder, bean.getCreateUser());

            MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
            BeanUtils.copyProperties(copyMesWorkOrder, workOrderLog, "id");
            orderLogList.add(workOrderLog);

            baseRDao.insert(copyMesWorkOrder);
            workOrderLogService.insertBatch(orderLogList);
            //写入接口表
            iMesWorkOrderToSapService.insert(bean);
            return bean;
        } else if (bean.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()) {
            //生成工单号
            genSerialNoModel = new GenSerialNoModel("ORDER_NO_JIS_UN_SORT");
        } else {
            //生成工单号
            genSerialNoModel = new GenSerialNoModel("ORDER_NO_BTO");
            Integer produceSeq = 0;
            if (null != result && result.getProduceSeq() != null) {
                produceSeq = result.getProduceSeq();
            }
            bean.setProduceSeq(produceSeq + 1);
            GenSerialNoModel orderSeqSerialNoModel = new GenSerialNoModel("ORDER_NO_BTO_SEQ");
            orderSeqSerialNoModel.setPartNo(bean.getPartNo());
            List<String> resultList = syncFuncService.syncSerialNo(orderSeqSerialNoModel, bean.getCreateUser(), bean.getOrganizeCode(), 1).getResultList();
            bean.setOrderSeq(Integer.parseInt(resultList.get(0)));
        }
        genSerialNoModel.setPartNo(bean.getPartNo());
        List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel, bean.getCreateUser(), bean.getOrganizeCode(), 1).getResultList();
        bean.setOrderNo(resultList.get(0));
        bean.setQty(bean.getQty());

        ConvertBean.serviceModelInitialize(bean, bean.getCreateUser());

        MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
        BeanUtils.copyProperties(bean, workOrderLog, "id");
        workOrderLogService.insert(workOrderLog);
        bean = baseRDao.insert(bean);
        //写入接口表
        iMesWorkOrderToSapService.insert(bean);
        return bean;
    }

    @Override
    public MesWorkOrder update(MesWorkOrder bean) {
        //回传sap
        boolean sendToSap = false;

        MesWorkOrder result = baseRDao.getById(bean.getId());
        //若工单类型为排序 则根据排序插入工单数量插入多行 工单数量为1
        if (result.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
            if(!Objects.equals(bean.getQty(), result.getQty())){
                MesException.throwMesBusiException(String.format("工单类型为【%s】 不需要修改数量",
                        MesExtEnumUtil.ORDER_TYPE.SORT.getDescription()));
            }
        }else{
            //工单数量只能比原来的大
            if(bean.getQty() < result.getQty()){
                MesException.throwMesBusiException(String.format("工单数量不能小于工单源数量【%s】",
                        result.getQty()));
            }
            //修改状态或者指令下发需要传给SAP
            if(!Objects.equals(bean.getQty(), result.getQty())){
                sendToSap = true;
            }
        }
        if(MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue() == bean.getWorkOrderStatus()){
            sendToSap = true;
        }
        //在加工的工单不能修改物料信息
        if(result.getWorkOrderStatus() >= MesExtEnumUtil.ORDER_STATUS.process.getValue()){
            if(!Objects.equals(bean.getPartNo(), result.getPartNo())){
                MesException.throwMesBusiException(String.format("工单状态为【%s】不允许修改物料",
                        MesExtEnumUtil.ORDER_TYPE.valueOfDescription(result.getWorkOrderStatus())));
            }
        }

        result.setQty(bean.getQty());
        result.setWorkOrderStatus(bean.getWorkOrderStatus());
        ConvertBean.serviceModelInitialize(bean, AuthUtil.getSessionUser().getUserName());

        MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
        BeanUtils.copyProperties(result, workOrderLog, "id");
        workOrderLogService.insert(workOrderLog);
        //关闭工单
        if(MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue() == bean.getWorkOrderStatus()){
            bean.setQty(0d);
            sendToSap = true;
        }
        //发送给SAP
        if(sendToSap){
            iMesWorkOrderToSapService.insert(bean);
        }

        baseRDao.update(result);
        return bean;
    }


    protected void setPackQueryBean(MesWorkOrder bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getOrderNo(),"orderNo",packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNameRdd(),"partNameRdd",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getPartNo(),"partNo",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustPartNo(),"custPartNo",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(),"workCenterCode",packBean);
        DdlPreparedPack.getNumEqualPack(bean.getWorkOrderStatus(),"workOrderStatus",packBean);
        DdlPreparedPack.getNumEqualPack(bean.getWorkOrderType(),"workOrderType",packBean);
        DdlPreparedPack.getStringBiggerPack(bean.getCreateDateTimeStart(),"createDatetime",packBean);
        DdlPreparedPack.getStringSmallerPack(bean.getCreateDateTimeEnd(),"createDatetime",packBean);
        DdlPreparedPack.getStringBiggerPack(bean.getStartTimeStart(),"planStartTime",packBean);
        DdlPreparedPack.getStringSmallerPack(bean.getStartTimeEnd(),"planStartTime",packBean);
        DdlPreparedPack.getStringBiggerPack(bean.getEndTimeStart(),"planEndTime",packBean);
        DdlPreparedPack.getStringSmallerPack(bean.getEndTimeEnd(),"planEndTime",packBean);
    }

    protected void onInsertBean(MesWorkOrder item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getWorkOrderType(), "工单类型不能为空");
        ValidatorBean.checkNotNull(item.getWorkCenterCode(), "产线不能为空");

        //ValidatorBean.checkNotNull(item.getShiftCode(), "班次不能为空");

        //校验标识
        //若工单类型为排序
        if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
            ValidatorBean.checkNotNull(item.getOrderFlag(), "标识不能为空");
            //若工单类型为排序 标识为 N E P S
            if (item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.S.getValue() &&
                    item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getValue() &&
                    item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue() &&
                    item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue()) {
                throw ImppExceptionBuilder.newInstance()
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                        .setErrorDetail(String.format("工单标识为【%s】请选择 【%s】、【%s】、【%s】、【%s】 标识",
                                item.getOrderFlag(), MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.S.getDescription()
                                , MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getDescription(),
                                MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getDescription(),
                                MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getDescription()))
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .build();
            }

        } else if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()) {
            ValidatorBean.checkNotNull(item.getOrderFlag(), "标识不能为空");
            if (item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getValue() &&
                    item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue()) {
                throw ImppExceptionBuilder.newInstance()
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                        .setErrorDetail(String.format("工单标识为【%s】请选择 【%s】、【%s】 标识",
                                item.getOrderFlag(), MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getDescription(),
                                MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getDescription()))
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .build();
            }
        }
        //物料名称不存在，从物料信息中获取
        if(StringUtil.isEmpty(item.getPartNameRdd())){
            MesPart mesPart = iMesPartService.getPartByPartNo(item.getPartNo(), item.getOrganizeCode());
            item.setPartNameRdd(mesPart.getPartName());
        }
        item.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.CREATE.getValue());
    }

    protected void onUpdateBean(MesWorkOrder item) {

    }

}
